<HTML><TITLE>Debug.c</TITLE><BODY><PRE>/*____________________________________________________________________________*\

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: Debug.c$
 * $Date: Sun Aug 10 06:37:32 1997$
 *
 Description:
	<A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> handler. Writes out debugging information. Place in
	important points of request processing to get context information
	of db values etc.

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include &ltstring.h>
#include &ltassert.h>
#include &ltstdio.h>

#include "Pi3API.h"


#define KEY_CONF_PI3EXPRESSION			"Pi3Expression"

/*____________________________________________________________________________*\
 *
 Description:
	Documentation
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
	<A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A>

Description:
	This handler writes a configurable debug message to the debug log.

Options:
&ltH5>Overview&lt/H5>
&ltTABLE BORDER=1>
&ltTH>Option
&ltTH>Default
&ltTH>Values
&ltTH>Short Description
&ltTH>Example(s)

&ltTR>
&ltTD><A HREF="/pidocs/Objects/Pi3API.html#Pi3Expression">Pi3Expression</A>
&ltTD>+
&ltTD>A <A HREF="/pidocs/Objects/Pi3API.html#Pi3Expression">Pi3Expression</A>
&ltTD>Expression to place in debug log
&ltTD>Pi3Expression="$c/* value of Content-Type */"

&lt/TABLE>
&ltSTRONG>-&lt/STRONG> in the &ltIT>default&lt/IT> indicates no default&ltBR>
&ltSTRONG>+&lt/STRONG> in the &ltIT>default&lt/IT> indicates the field is mandatory&ltBR>

&ltH4>Description of Options&lt/H4>
&ltH5>
	<A HREF="/pidocs/Objects/Pi3API.html#Pi3Expression">Pi3Expression</A>
&lt/H5>
<A HREF="/pidocs/Objects/Pi3API.html#Pi3Expression">Pi3Expression</A> to write in the debug log.

Phase:
	All phases

Returns:
	PIAPI_CONTINUE

Note:
Example:
	&ltPRE>
	&ltObject&gt
		Name <A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A>
		Class DebugClass
	&lt/Object&gt

	&ltObject&gt
		...
		Handle <A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> Pi3Expression="$c/* value of Content-Type */"
		...
	&lt/Object&gt
	&lt/PRE>
/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
struct _Debug
{
	<A HREF="/pidocs/Objects/Pi3API.html#Pi3Expression">Pi3Expression</A> *pInfo;
};
typedef struct _Debug Debug;

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Debug_fnParameter( void *pData, const char *pVar, const char *pVal,
	const char *pWhere )
{
	PIObject *pObject = (PIObject *)pData;
	<A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> *pDebug = (<A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> *)<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getUserData">PIObject_getUserData</A>( pObject );
	(void)pDebug;
	assert( pVar );
	assert( pVal );

	if ( !<A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_stricmp">PIUtil_stricmp</A>( pVar, KEY_CONF_PI3EXPRESSION ) )
		{
		Pi3String *pError;
		<A HREF="/pidocs/Objects/Pi3API.html#Pi3Expression">Pi3Expression</A> *pExpr;
		if ( pDebug->pInfo )
			{
			<A HREF="/pidocs/APISpec/Pi2API.html#PILog_addMessage">PILog_addMessage</A>( <A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getDB">PIObject_getDB</A>( pObject ),
				<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getConfigurationDB">PIObject_getConfigurationDB</A>( pObject ),
				PILOG_ERROR, "Debug: 'Pi3Expression' should only be \
specified once.", pWhere );
			return 0;
			};
		pError = <A HREF="/pidocs/APISpec/Pi3API.html#Pi3String_new">Pi3String_new</A>( 0 );
		pExpr = <A HREF="/pidocs/APISpec/Pi3API.html#Pi3Expression_new">Pi3Expression_new</A>( pVal, 0, pError );
		if ( !pExpr )	
			{
			<A HREF="/pidocs/APISpec/Pi2API.html#PILog_addMessage">PILog_addMessage</A>( <A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getDB">PIObject_getDB</A>( pObject ),
				<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getConfigurationDB">PIObject_getConfigurationDB</A>( pObject ),
				PILOG_ERROR, "%sDebug: %s", pError, <A HREF="/pidocs/APISpec/Pi3API.html#Pi3String_getPtr">Pi3String_getPtr</A>( pError ) );
			<A HREF="/pidocs/APISpec/Pi3API.html#Pi3String_delete">Pi3String_delete</A>( pError );
			return 0;
			};
		<A HREF="/pidocs/APISpec/Pi3API.html#Pi3String_delete">Pi3String_delete</A>( pError );
		pDebug->pInfo = pExpr;
		}
	else
		{
		/* --- Configuration error --- */
		<A HREF="/pidocs/APISpec/Pi2API.html#PILog_addMessage">PILog_addMessage</A>( <A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getDB">PIObject_getDB</A>( pObject ),
			<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getConfigurationDB">PIObject_getConfigurationDB</A>( pObject ),
			PILOG_ERROR, "%sUnknown directive '%s'.", pWhere, pVar );
		return 0;
		};
	return 1;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Debug_init( <A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> *pDebug, PIObject *pObject, int iArgc,
	const char *ppArgv[] )
{
	int iRet = <A HREF="/pidocs/APISpec/Pi2API.html#PIObject_readParameters">PIObject_readParameters</A>( pObject, iArgc, ppArgv,
		Debug_fnParameter, pObject );

	/* --- set defaults --- */
	if ( iRet )
		{
		if ( !pDebug->pInfo )	
			{
			<A HREF="/pidocs/APISpec/Pi2API.html#PILog_addMessage">PILog_addMessage</A>(
				<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getDB">PIObject_getDB</A>( pObject ),	
				<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getConfigurationDB">PIObject_getConfigurationDB</A>( pObject ),
				PILOG_ERROR, "%s", "Debug: 'Pi3Expression' not specified." );
			return 0;
			};
		};

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Debug_handle( PIHTTP *pPIHTTP, <A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> *pDebug )
{
	/* ---
	Buffer for text 
	--- */
	enum { BUF_SIZE=2047 };
	char szBuf[BUF_SIZE+1];
	char *pBuf = szBuf;
	int iLen;

	assert( pDebug );
	assert( pDebug->pInfo );

	iLen = <A HREF="/pidocs/APISpec/Pi3API.html#Pi3Expression_write">Pi3Expression_write</A>( pDebug->pInfo, pPIHTTP, 0, pBuf, BUF_SIZE );
	if ( iLen==-1 )
		{ return PIAPI_ERROR; };
	if ( iLen>BUF_SIZE )
		{
		pBuf = <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_malloc">PIUtil_malloc</A>( iLen+1 );
		<A HREF="/pidocs/APISpec/Pi3API.html#Pi3Expression_write">Pi3Expression_write</A>( pDebug->pInfo, pPIHTTP, 0, pBuf, iLen );
		};

	pBuf[iLen] = '\0';
	<A HREF="/pidocs/APISpec/Pi3API.html#HTTPCore_logDebug">HTTPCore_logDebug</A>( DBG_HIGH, pBuf );

	if ( pBuf!=szBuf )
		{
		<A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_free">PIUtil_free</A>( pBuf );
		};

	/* ---
	Return continue to allow other handlers to be invoked for this phase
	--- */
	return PIAPI_CONTINUE;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int Debug_onClassLoad( PIClass_LoadAction eLoad, int i,
	const char *a[] )
{
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int Debug_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	<A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> *pDebug = <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_malloc">PIUtil_malloc</A>( sizeof( <A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> ) );
	memset( pDebug, 0, sizeof( <A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> ) );
	<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_setUserData">PIObject_setUserData</A>( pObj, pDebug );
	if ( !Debug_init( pDebug, pObj, iArgc, ppArgv ) )
		{
		return PIAPI_ERROR;
		};
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int Debug_copyConstructor( PIObject *o, int i, const char *a[] )
{
	return PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int Debug_execute( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	PIHTTP *pPIHTTP;

	if ( iArgc&lt1 )
		{ return PIAPI_ERROR; };

	pPIHTTP = (PIHTTP *)*ppArgv;

	return Debug_handle( pPIHTTP, (<A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> *)<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getUserData">PIObject_getUserData</A>( pObj ) ); 
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int Debug_destructor( PIObject *pObj, int i, const char *a[] )
{
	<A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> *pDebug = (<A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A> *)<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getUserData">PIObject_getUserData</A>( pObj );
	if ( pDebug->pInfo )
		{ <A HREF="/pidocs/APISpec/Pi3API.html#Pi3Expression_delete">Pi3Expression_delete</A>( pDebug->pInfo ); };
	<A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_free">PIUtil_free</A>( pDebug );
	return PIAPI_COMPLETED;
}

#if 0
/*___+++CNF_BEGIN+++___*/
	&ltClass>
		Name DebugClass
		Type LogicExtension
		Library Plugins
		OnClassLoad Debug_onClassLoad
		Constructor Debug_constructor
		CopyConstructor Debug_copyConstructor
		Destructor Debug_destructor
		Execute Debug_execute
	&lt/Class>

	&ltObject>
		Name <A HREF="/pidocs/Objects/Plugins.html#Debug">Debug</A>
		Class DebugClass
	&lt/Object>

/*___+++CNF_END+++___*/
#endif

</PRE></BODY></HTML>