<HTML><TITLE>UserDir.c</TITLE><BODY><PRE>/*____________________________________________________________________________*\

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: UserDir.c$
 * $Date: Sun Aug 10 06:37:33 1997$
 *
 Description:
	User directory mapping.

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include &ltstring.h>
#include &ltassert.h>

#include "Pi3API.h"

#define KEY_CONF_USERDIRECTORYROOT		"UserDirectoryRoot"
#define KEY_CONF_HTMLDIRECTORY			"HTMLDirectory"

/*____________________________________________________________________________*\
 *
 Description:
	Documentation
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
	<A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A>

Description:
	Remap user directories.

	Example
&ltB>&ltPRE>
	UserDirectoryRoot="/home/"
	HTMLDirectory="/public_html"

	/~jroy/hello.html --> /home/jroy/public_html/hello.html

&lt/PRE>&lt/B>

Options:
&ltH5>Overview&lt/H5>
&ltTABLE BORDER=1>
&ltTH>Option
&ltTH>Default
&ltTH>Values
&ltTH>Short Description
&ltTH>Example(s)

&ltTR>
&ltTD>UserDirectoryRoot
&ltTD>+
&ltTD>A directory path
&ltTD>Root of user directories
&ltTD>UserDirectoryRoot="/home/"

&ltTR>
&ltTD>HTMLDirectory
&ltTD>+
&ltTD>A directory name
&ltTD>User directory with web files
&ltTD>HTMLDirectory="/public_html"

&lt/TABLE>
&ltSTRONG>-&lt/STRONG> in the &ltIT>default&lt/IT> indicates no default&ltBR>
&ltSTRONG>+&lt/STRONG> in the &ltIT>default&lt/IT> indicates the field is mandatory&ltBR>

&lt!--
Dont forget the slashes
&ltH4>Description of Options&lt/H4>
&ltH5>
	<A HREF="/pidocs/Objects/Pi3API.html#Pi3Expression">Pi3Expression</A>
&lt/H5>
<A HREF="/pidocs/Objects/Pi3API.html#Pi3Expression">Pi3Expression</A> to write in the debug log.
-->

Phase:
	MAPPING

Returns:
	PIAPI_COMPLETED if the directory was mapped, otherwise PIAPI_CONTINUE.

Note:
Example:
	&ltPRE>
	&ltObject&gt
		Name <A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A>
		Class UserDirectoryClass
		UserDirectoryRoot "/home/"
		HTMLDirectory "/public_html"
	&lt/Object&gt

	&ltObject&gt
		...
		Handle <A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A>
		...
	&lt/Object&gt
	&lt/PRE>
/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
struct _UserDirectory
{
	char *pRoot;
	int iRootLen;
	char *pDirectory;
	int iDirectoryLen;
};
typedef struct _UserDirectory UserDirectory;

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int UserDirectory_fnParameter( void *pData, const char *pVar, const char *pVal,
	const char *pWhere )
{
	PIObject *pObject = (PIObject *)pData;
	<A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> *pUserDirectory = (<A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> *)<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getUserData">PIObject_getUserData</A>( pObject );
	(void)pUserDirectory;
	assert( pVar );
	assert( pVal );

	if ( !<A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_stricmp">PIUtil_stricmp</A>( pVar, KEY_CONF_USERDIRECTORYROOT) )
		{
		pUserDirectory->pRoot = <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_strdup">PIUtil_strdup</A>( pVal );
		pUserDirectory->iRootLen = strlen( pUserDirectory->pRoot );
		}
	else if ( !<A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_stricmp">PIUtil_stricmp</A>( pVar, KEY_CONF_HTMLDIRECTORY) )
		{
		pUserDirectory->pDirectory = <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_strdup">PIUtil_strdup</A>( pVal );
		pUserDirectory->iDirectoryLen = strlen( pUserDirectory->pDirectory );
		}
	else
		{
		/* --- Configuration error --- */
		<A HREF="/pidocs/APISpec/Pi2API.html#PILog_addMessage">PILog_addMessage</A>( <A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getDB">PIObject_getDB</A>( pObject ),
			<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getConfigurationDB">PIObject_getConfigurationDB</A>( pObject ),
			PILOG_ERROR, "UserDirectory: %sUnknown directive '%s'.",
			pWhere, pVar );
		return 0;
		};
	return 1;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int UserDirectory_init( <A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> *pUserDirectory, PIObject *pObject, int iArgc,
	const char *ppArgv[] )
{
	int iRet = <A HREF="/pidocs/APISpec/Pi2API.html#PIObject_readParameters">PIObject_readParameters</A>( pObject, iArgc, ppArgv,
		UserDirectory_fnParameter, pObject );

	/* --- set defaults --- */
	if ( iRet )
		{
		if ( !pUserDirectory->pRoot )	
			{
			<A HREF="/pidocs/APISpec/Pi2API.html#PILog_addMessage">PILog_addMessage</A>(
				<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getDB">PIObject_getDB</A>( pObject ),	
				<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getConfigurationDB">PIObject_getConfigurationDB</A>( pObject ),
				PILOG_ERROR, "%s", "UserDirectory: 'UserDirectoryRoot' \
not specified." );
			iRet = 0;
			}
		else if ( !pUserDirectory->pDirectory )	
			{
			<A HREF="/pidocs/APISpec/Pi2API.html#PILog_addMessage">PILog_addMessage</A>(
				<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getDB">PIObject_getDB</A>( pObject ),	
				<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getConfigurationDB">PIObject_getConfigurationDB</A>( pObject ),
				PILOG_ERROR, "%s", "UserDirectory: 'HTMLDirectory' \
not specified." );
			iRet = 0;
			};
		};

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int UserDirectory_handle( PIHTTP *pPIHTTP, <A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> *pUserDirectory )
{
	const char *pPath;
	int i;
	enum { BUF_SIZE=2047 };
	char szBuffer[BUF_SIZE+1];
	char *pBuffer = szBuffer;
	int iSize;

	assert( pUserDirectory );
	assert( pUserDirectory->pRoot );
	assert( pUserDirectory->pDirectory );

	/* ---
	Get Path
	--- */
	pPath = (const char *)<A HREF="/pidocs/APISpec/Pi2API.html#PIDB_lookup">PIDB_lookup</A>( pPIHTTP->pResponseDB, PIDBTYPE_STRING,
		KEY_INT_PATH, 0 );

	/* ---
	See if this path is not a user path
	--- */
	if ( !pPath || *pPath!='/' || pPath[1]!='~' )
		{
		return PIAPI_CONTINUE;
		};

	/* ---
	Scan to first '/' character
	--- */
	pPath = &( pPath[2] );
	for( i=0; pPath[i] && pPath[i]!='/'; i++  );

	/* ---
	Write in root 
	--- */
	iSize = BUF_SIZE;
	if ( !iSize ) { goto buffer_error; };
	strncpy( pBuffer, pUserDirectory->pRoot, iSize );
	iSize -= pUserDirectory->iRootLen;
	pBuffer = &( pBuffer[pUserDirectory->iRootLen] );

	/* ---
	Then user name
	--- */
	if ( iSize&lti ) { goto buffer_error; };
	if ( i )
		{
		strncat( pBuffer, pPath, i );
		iSize -= i;
		pBuffer = &( pBuffer[i] );
		}; 

	/* ---
	Then HTML directory
	--- */
	if ( iSize&ltpUserDirectory->iDirectoryLen ) { goto buffer_error; };
	if ( pUserDirectory->iDirectoryLen )
		{
		strcat( pBuffer, pUserDirectory->pDirectory );
		iSize -= pUserDirectory->iDirectoryLen;
		pBuffer = &( pBuffer[pUserDirectory->iDirectoryLen] );
		};

	/* ---
	Then the path that followed the user directory
	--- */
	pPath = &( pPath[i] );
	i = strlen( pPath );
	if ( iSize&lti ) { goto buffer_error; };
	if ( i )
		{
		strcat( pBuffer, pPath );
		};

	/* ---
	Done, replace path
	--- */
	<A HREF="/pidocs/APISpec/Pi2API.html#PIDB_replace">PIDB_replace</A>( pPIHTTP->pResponseDB, PIDBTYPE_STRING, KEY_INT_PATH,
		szBuffer, 0 );

	/* 
	** NOTE: Verify that path is world readable? (UNIX only)
	*/

	return PIAPI_COMPLETED;

buffer_error:
	
	<A HREF="/pidocs/APISpec/Pi3API.html#HTTPCore_logError">HTTPCore_logError</A>( pPIHTTP, "UserDirectory: Internal error remapping \
to user directory. Path components too long for internal buffer." );

	/* ---
	Internal redirect to error handler
	--- */
	return <A HREF="/pidocs/APISpec/Pi3API.html#HTTPUtil_doHTTPError">HTTPUtil_doHTTPError</A>( pPIHTTP, ST_INTERNALERROR );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int UserDirectory_onClassLoad( PIClass_LoadAction eLoad, int i,
	const char *a[] )
{
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int UserDirectory_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	<A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> *pUserDirectory = <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_malloc">PIUtil_malloc</A>( sizeof( <A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> ) );
	memset( pUserDirectory, 0, sizeof( <A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> ) );
	<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_setUserData">PIObject_setUserData</A>( pObj, pUserDirectory );
	if ( !UserDirectory_init( pUserDirectory, pObj, iArgc, ppArgv ) )
		{
		return PIAPI_ERROR;
		};
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int UserDirectory_copyConstructor( PIObject *o, int i, const char *a[] )
{
	return PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int UserDirectory_execute( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	PIHTTP *pPIHTTP;

	if ( iArgc&lt1 )
		{ return PIAPI_ERROR; };

	pPIHTTP = (PIHTTP *)*ppArgv;

	if ( pPIHTTP->ciPhase!=PH_MAPPING )
		{ return PIAPI_ERROR; };

	return UserDirectory_handle( pPIHTTP, 
		(<A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> *)<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getUserData">PIObject_getUserData</A>( pObj ) ); 
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int UserDirectory_destructor( PIObject *pObj, int i, const char *a[] )
{
	<A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> *pUserDirectory = (<A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A> *)<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getUserData">PIObject_getUserData</A>( pObj );
	if ( pUserDirectory->pRoot )
		{ <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_free">PIUtil_free</A>( pUserDirectory->pRoot ); };
	if ( pUserDirectory->pDirectory )
		{ <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_free">PIUtil_free</A>( pUserDirectory->pDirectory ); };
	<A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_free">PIUtil_free</A>( pUserDirectory );
	return PIAPI_COMPLETED;
}

#if 0
/*___+++CNF_BEGIN+++___*/
	&ltClass>
		Name UserDirectoryClass
		Type LogicExtension
		Library Plugins
		OnClassLoad UserDirectory_onClassLoad
		Constructor UserDirectory_constructor
		CopyConstructor UserDirectory_copyConstructor
		Destructor UserDirectory_destructor
		Execute UserDirectory_execute
	&lt/Class>

	&ltObject>
		Name <A HREF="/pidocs/Objects/Plugins.html#UserDirectory">UserDirectory</A>
		Class UserDirectoryClass
	&lt/Object>

/*___+++CNF_END+++___*/
#endif

</PRE></BODY></HTML>